from reader_writer_lock.Base import BaseLock


class ReadLock(BaseLock):
    def __init__(self, lock, cv_read, cv_write, active_read, active_write, waiting_read, waiting_write, options=0):
        super().__init__(lock, cv_read)

        self.cv_read = cv_read
        self.active_read = active_read
        self.active_write = active_write
        self.waiting_read = waiting_read
        self.waiting_write = waiting_write

        self.cv_write = cv_write
        self.waiting_cond = self.construct_waiting_cond(options)

    def construct_waiting_cond(self, options):
        if options == 2:
            return self.prefer_writing_cond

        return self.normal_cond

    def prefer_writing_cond(self):
        return self.active_write.value() > 0 or self.waiting_write.value() > 0

    def normal_cond(self):
        return self.active_write.value() > 0

    def acquire(self):
        with self.global_lock:
            self.waiting_read.incre()

            while self.waiting_cond():
                self.cv_read.wait()

            self.waiting_read.decre()
            self.active_read.incre()

    def release(self):
        with self.global_lock:
            self.active_read.decre()
            self.cv_write.notify_all()



class BaseLock:
    def __init__(self, lock, condition_variable):
        self.global_lock = lock
        self.cv = condition_variable

    def acquire(self):
        pass

    def release(self):
        pass

    def __enter__(self):
        self.acquire()
        return self

    def __exit__(self, type, value, traceback):
        self.release()


class BaseInteger:
    def __init__(self):
        pass

    def incre(self):
        pass

    def decre(self):
        pass

    def value(self):
        pass



from setuptools import setup, find_packages
import os

here = os.path.abspath(os.path.dirname(__file__))

with open("README.md", "r") as fh:
    long_description = fh.read()

VERSION = '0.0.3'
DESCRIPTION = 'Reader Writer Lock'

# Setting up
setup(
    name="reader_writer_locks",
    version=VERSION,
    author="Viet Le",
    author_email="minhviet810business@gmail.com",
    license='MIT',
    description=DESCRIPTION,
    long_description_content_type='text/markdown',
    long_description=long_description,
    packages=find_packages(),
    install_requires=[],
    keywords=['concurrency', 'multithreading', 'multiprocessing'],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    project_urls={
        'Source': 'https://github.com/leminhviett/ReaderWriterLock'
    }
)



from reader_writer_lock.ReadLock import ReadLock
from reader_writer_lock.WriteLock import WriteLock
from reader_writer_lock.Base import BaseInteger
from threading import RLock, Condition


class Integer(BaseInteger):
    def __init__(self, init):
        super(Integer, self).__init__()
        self._value = init

    def value(self):
        return self._value

    def incre(self):
        self._value += 1

    def decre(self):
        self._value -= 1


# factory method
class FactoryLock:
    '''
    options:
        0: normal
        1: read priority
        2: write priority
    '''

    def __init__(self, options):
        if options > 2 or options < 0:
            raise Exception("Options must be in [0, 1, 2]")

        self.global_lock = RLock()
        self.active_read = Integer(0)
        self.active_write = Integer(0)
        self.waiting_read = Integer(0)
        self.waiting_write = Integer(0)
        self.cv_read = Condition(self.global_lock)
        self.cv_write = Condition(self.global_lock)

        self.read_lock = ReadLock(self.global_lock, self.cv_read, self.cv_write, self.active_read, self.active_write,
                                  self.waiting_read, self.waiting_write, options)
        self.write_lock = WriteLock(self.global_lock, self.cv_read, self.cv_write, self.active_read, self.active_write,
                                    self.waiting_read, self.waiting_write, options)

    def get_read_lock(self):
        return self.read_lock

    def get_write_lock(self):
        return self.write_lock
